# Contributing to PsiZ

When contributing to this repository, please first discuss the change you wish to make via issue, email, or any other method with the owners of this repository before making a change.

Please note we have a code of conduct, please follow it in all your interactions with the project.

PsiZ's scope is restricted to computational modeling of human behavioral data. This includes similarity ratings, similarity rankings, pile-sorts, and categorization of stimuli. Not all of this functionality is implemented. Contributions that support this functionality are welcome.

## Issues

* Please tag your issue with `bug`, `enhancement`, or `question` to help us effectively respond.
* Please include the versions of TensorFlow, TensorFlow-Probability and PsiZ you are running.
* Please provide the command line or code you ran as well as the log output.

## Pull Requests

Please send in fixes and feature additions through Pull Requests.

## Testing

* PsiZ uses `pytest` for testing, `pytest-cov` for coverage analytics, and `tox` for testing multiple python versions. These packages should be installed separately by the tester, but most of these packages can be installed at the same time as the core dependencies of psiz using the option `pip install "psiz[test]"`.
* A coverage report can be generated using `pytest --cov`
* An xml coverage report for uploading to codecov can be generated using `py.test --cov-report=xml --cov=psiz tests/`
* Tests that take more than a few seconds are marked as `slow` and can be skipped by running `pytest -m "not slow"`.
* All pytest markers must be registered in `pytest.ini`, unregistered markers will generate an error.

## Versioning
* Versions are released following [Semantic Versioning 2.0.0](https://semver.org/) which follows the MAJOR.MINOR.PATCH format.
* Developers should create feature branches that branch from a particular "major-minor branch" `rX.Y`. The major-minor branches serve as a development branches. Note that a new branch is NOT created for each patch version.
* In preparation for a release, a coverage report must be created and uploaded to codecov.io.
* All releases are tagged using the format `vX.Y.Z`, thus a given release branch can have multiple tags that differ by patch number.
* Only tagged releases on a major-minor branch are merged with `main`. Thus, the `main` branch always represents the latest stable version.

# Additional Guidance

PsiZ closely adheres to TensorFlow and Keras idioms. Model components are implemented as layers. Custom Keras objects are placed in `psiz.keras` and intentionally mirror the module structure of `tensorflow.keras` in order to leverage developers pre-existing knowledge of TensorFlow's organization.

## Module: trials
* Code to the appropriate interface.

### Trials
* All trial objects have a n_trial attribute.
* All trial objects have a stimulus_set attribute.

### Observations
* requires as_datatset() method

## Module: models
* take an appropriate Dataset as an argument
* for get_config(): follow the {'class_name': str, 'config': dict} pattern

## Contributing to psiz.keras

Since the models used in this package are prone to find sub-optimal solutions, multiple restarts are necessary. This results in a point of divergence from typical TensorFlow projects. TensorFlow does not provide a pre-packaged strategy for performing multiple restarts of a model. To accommodate this need, some basic restart functionality is provided by the Restarter class.

Register the class via the decorator @tf.keras.utils.register_keras_serializable in order to facilitate model loading.
